# noble

[![Gitter](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/sandeepmistry/noble?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)

A node.js BLE (Bluetooth low energy) central module.

Want to implement a peripheral? Checkout [bleno](https://github.com/sandeepmistry/bleno)

__Note:__ Mac OS X and Linux are currently the only supported OSes. Other platforms may be developed later on (see Roadmap below).

## Prerequisites

### OS X

 * install [Xcode](https://itunes.apple.com/ca/app/xcode/id497799835?mt=12)

### Linux (Ubuntu)

 * Kernel version 3.6 or above
 * ```libbluetooth-dev```

#### Ubuntu/Debian/Raspbian

```sh
sudo apt-get install bluetooth bluez-utils libbluetooth-dev
```

#### Intel Edison

See [Configure Intel Edison for Bluetooth LE (Smart) Development](http://rexstjohn.com/configure-intel-edison-for-bluetooth-le-smart-development/)

## Install

```sh
npm install noble
```

## Usage

```javascript
var noble = require('noble');
```

### Actions

#### Start scanning

```javascript
noble.startScanning(); // any service UUID, no duplicates


noble.startScanning([], true); // any service UUID, allow duplicates


var serviceUUIDs = ["<service UUID 1>", ...]; // default: [] => all
var allowDuplicates = <false|true>; // default: false

noble.startScanning(serviceUUIDs, allowDuplicates); // particular UUID's
```

#### Stop scanning

```javascript
noble.stopScanning();
```

#### Peripheral

##### Connect

```javascript
peripheral.connect([callback(error)]);
```

##### Disconnect or cancel pending connection

```javascript
peripheral.disconnect([callback(error)]);
```

##### Update RSSI

```javascript
peripheral.updateRssi([callback(error, rssi)]);
```

##### Discover services

```javascript
peripheral.discoverServices(); // any service UUID

var serviceUUIDs = ["<service UUID 1>", ...];
peripheral.discoverServices(serviceUUIDs[, callback(error, services)]); // particular UUID's
```

##### Discover all services and characteristics

```javascript
peripheral.discoverAllServicesAndCharacteristics([callback(error, services, characteristics));
```

##### Discover some services and characteristics

```javascript
var serviceUUIDs = ["<service UUID 1>", ...];
var characteristicUUIDs = ["<characteristic UUID 1>", ...];
peripheral.discoverSomeServicesAndCharacteristics(serviceUUIDs, characteristicUUIDs, [callback(error, services, characteristics));
```
#### Service

##### Discover included services

```javascript
service.discoverIncludedServices(); // any service UUID

var serviceUUIDs = ["<service UUID 1>", ...];
service.discoverIncludedServices(serviceUUIDs[, callback(error, includedServiceUuids)]); // particular UUID's
```

##### Discover characteristics

```javascript
service.discoverCharacteristics() // any characteristic UUID

var characteristicUUIDs = ["<characteristic UUID 1>", ...];
service.discoverCharacteristics(characteristicUUIDs[, callback(error, characteristics)]); // particular UUID's
```

#### Characteristic

##### Read

```javascript
characteristic.read([callback(error, data)]);
```

##### Write

```javascript
characteristic.write(data, notify[, callback(error)]); // data is a buffer, notify is true|false
```

##### Broadcast

```javascript
characteristic.broadcast(broadcast[, callback(error)]); // broadcast is true|false
```

##### Notify

```javascript
characteristic.notify(notify[, callback(error)]); // notify is true|false
```

  * use for characteristics with notifiy or indicate properties

##### Discover descriptors

```javascript
characteristic.discoverDescriptors([callback(error, descriptors)]);
```

##### Read value

```javascript
descriptor.readValue([callback(error, data)]);
```

##### Write value

```javascript
descriptor.writeValue(data[, callback(error)]); // data is a buffer
```

#### Handle

##### Read

```javascript
peripheral.readHandle(handle, callback(error, data));
```

##### Write

```javascript
peripheral.writeHandle(handle, data, withoutResponse, callback(error));
```

### Events

#### Adapter state change

```javascript
state = <"unknown" | "resetting" | "unsupported" | "unauthorized" | "poweredOff" | "poweredOn">

noble.on('stateChange', callback(state));
```

#### Scan started:

```javascript
noble.on('scanStart', callback);
```

#### Scan stopped

```javascript
noble.on('scanStop', callback);
```

#### Peripheral discovered:

```javascript
peripheral = {
  uuid: "<uuid>",
  advertisement: {
    localName: "<name>",
    txPowerLevel: <int>,
    serviceUuids: ["<service UUID>", ...],
    manufacturerData: <Buffer>,
    serviceData: [
        {
            uuid: "<service UUID>"
            data: <Buffer>
        },
        ...
    ]
  },
  rssi: <rssi>
};

noble.on('discover', callback(peripheral));
```

#### Peripheral

##### Connected

```javascript
peripheral.on('connect', callback);
```

##### Disconnected:

```javascript
peripheral.on('disconnect', callback);
```

##### RSSI update

```javascript
peripheral.on('rssiUpdate', callback(rssi));
```

##### Services discovered

```javascript
peripheral.on('servicesDiscover', callback(services));
```

#### Service

##### Included services discovered

```javascript
service.on('includedServicesDiscover', callback(includedServiceUuids));
```

##### Characteristics discovered

```javascript
characteristic = {
  uuid: "<uuid>",
   // properties: 'broadcast', 'read', 'writeWithoutResponse', 'write', 'notify', 'indicate', 'authenticatedSignedWrites', 'extendedProperties'
  properties: [...]
};

service.on('characteristicsDiscover', callback(characteristics));
```

#### Characteristic

##### Data

Emitted when characteristic read has completed, result of ```characteristic.read(...)``` or characteristic value has been updated by peripheral via notification or indication.

```javascript
characteristic.on('data', callback(data, isNotification));

characteristic.on('read', callback(data, isNotification)); // legacy
```

##### Write

Emitted when characteristic write has completed, result of ```characteristic.write(...)```.

```javascript
characteristic.on('write', withoutResponse, callback());
```

##### Broadcast

Emitted when characteristic broadcast state changes, result of ```characteristic.broadcast(...)```.

```javascript
characteristic.on('broadcast', callback(state));
```

##### Notify

Emitted when characteristic notification state changes, result of ```characteristic.notify(...)```.

```javascript
characteristic.on('notify', callback(state));
```

##### Descriptors discovered

```javascript
descriptor = {
  uuid: '<uuid>'
};

characteristic.on('descriptorsDiscover', callback(descriptors));
```

#### Descriptor

##### Value read

```javascript
descriptor.on('valueRead', data);
```

##### Value write

```javascript
descriptor.on('valueWrite');
```

## Running on Linux

### Running without root/sudo

Run the following command in the directory you ran ```npm install``` from:

```sh
find -path '*noble*Release/hci-ble' -exec sudo setcap cap_net_raw+eip '{}' \;
```

This grants noble's ```hci-ble``` binary ```cap_net_raw``` privileges, so it can start/stop scanning for BLE devices.

__Note:__ The above command requires ```setcap``` to be installed, it can be installed using the following:

 * apt: ```sudo apt-get install libcap2-bin```
 * yum: ```su -c \'yum install libcap2-bin\'```

### Multiple Adapters

```hci0``` is used by default to override set the ```NOBLE_HCI_DEVICE_ID``` environment variable to the interface number.

Example, specify ```hci1```:

```sh
sudo NOBLE_HCI_DEVICE_ID=1 node <your file>.js
```

### Reporting all HCI events

By default noble waits for both the advertisement data and scan response data for each Bluetooth address. If your device does not use scan response the following enviroment variable can be used to bypass it.


```sh
sudo NOBLE_REPORT_ALL_HCI_EVENTS=1 node <your file>.js
```

## Roadmap (TODO)

 * Mac OS X:
   * ~~Adapter state (unknown | reseting | unsupported | unauthorized | off | on)~~
   * ~~Scan~~
      * ~~startScanning~~
         * ~~service UUID's~~
         * ~~allow duplicates~~
      * ~~stopScanning~~
   * ~~Peripheral~~
     * ~~discovered~~
     * ~~connect~~
     * ~~disconnect/cancel connect~~
     * ~~update RSSI~~
     * ~~services~~
         * ~~discover~~
         * ~~disover included~~
         * ~~discover characteristics for services~~
     * ~~characteristics~~
         * ~~read~~
         * ~~write~~
         * ~~set broadcast value~~
         * ~~set notify/indicate value~~
         * ~~descriptors~~
             * ~~discover~~
             * ~~read~~
             * ~~write~~
     * ~~handle~~
         * ~~read~~
         * ~~write~~
             * ~~with response~~
             * without response
   * error handling

 * Linux
   * ~~Adapter state (unsupported | unauthorized | off | on)~~
   * ~~Scan~~
      * ~~startScanning~~
         * ~~service UUID's~~
         * ~~allow duplicates~~
      * ~~stopScanning~~
   * ~~Peripheral~~
     * ~~discovered~~
     * ~~connect~~
         * ~~public address~~
         * ~~random address~~
     * ~~disconnect/cancel connect~~
     * ~~update RSSI~~
     * ~~services~~
         * ~~discover~~
             * ~~filter by uuid~~
         * ~~discover included~~
         * ~~discover characteristics for services~~
             * ~~filter by uuid~~
     * ~~characteristics~~
         * ~~read~~
         * ~~write~~
         * ~~set broadcast value~~
         * ~~set notify/indicate value~~
         * ~~descriptors~~
             * ~~discover~~
             * ~~read~~
             * ~~write~~
     * ~~handle~~
         * ~~read~~
         * ~~write~~
             * ~~with response~~
             * ~~without response~~
   * error handling
 * Windows
   * TDB (most likely Windows 8 only)

## Useful Links

 * [Bluetooth Development Portal](http://developer.bluetooth.org)
   * [GATT Specifications](http://developer.bluetooth.org/gatt/Pages/default.aspx)
 * [Bluetooth: ATT and GATT](http://epx.com.br/artigos/bluetooth_gatt.php)

## License

Copyright (C) 2015 Sandeep Mistry <sandeep.mistry@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

[![Analytics](https://ga-beacon.appspot.com/UA-56089547-1/sandeepmistry/noble?pixel)](https://github.com/igrigorik/ga-beacon)

